@@ -79,26 +79,15 @@ func commentBitbucketCmd(ctx *config.RunContext) *cobra.Command {
 
 			paths, _ := cmd.Flags().GetStringArray("path")
 
-			commentOut, err := buildCommentOutput(cmd, ctx, paths, output.MarkdownOptions{
+			commentOut, commentErr := buildCommentOutput(cmd, ctx, paths, output.MarkdownOptions{
 				WillUpdate:          prNumber != 0 && behavior == "update",
 				WillReplace:         prNumber != 0 && behavior == "delete-and-new",
 				IncludeFeedbackLink: !ctx.Config.IsSelfHosted(),
 				OmitDetails:         extra.OmitDetails,
 				BasicSyntax:         true,
 			})
-			var policyFailure output.PolicyCheckFailures
-			var guardrailFailure output.GuardrailFailures
-			var tagPolicyFailure *output.TagPolicyCheck
-			if err != nil {
-				if v, ok := err.(output.PolicyCheckFailures); ok {
-					policyFailure = v
-				} else if v, ok := err.(output.GuardrailFailures); ok {
-					guardrailFailure = v
-				} else if v, ok := err.(output.TagPolicyCheck); ok {
-					tagPolicyFailure = &v
-				} else {
-					return err
-				}
+			if isErrorUnhandled(commentErr) {
+				return commentErr
 			}
 
 			dryRun, _ := cmd.Flags().GetBool("dry-run")
@@ -133,17 +122,7 @@ func commentBitbucketCmd(ctx *config.RunContext) *cobra.Command {
 				cmd.Println("Comment not posted to Bitbucket (--dry-run was specified)")
 			}
 
-			if policyFailure != nil {
-				return policyFailure
-			}
-			if guardrailFailure != nil {
-				return guardrailFailure
-			}
-			if tagPolicyFailure != nil {
-				return tagPolicyFailure
-			}
-
-			return nil
+			return commentErr
 		},
 	}
 
