@@ -191,7 +191,6 @@ func newRootCmd(ctx *config.RunContext) *cobra.Command {
 	rootCmd.AddCommand(configureCmd(ctx))
 	rootCmd.AddCommand(diffCmd(ctx))
 	rootCmd.AddCommand(breakdownCmd(ctx))
-	rootCmd.AddCommand(scanCommand(ctx))
 	rootCmd.AddCommand(outputCmd(ctx))
 	rootCmd.AddCommand(uploadCmd(ctx))
 	rootCmd.AddCommand(commentCmd(ctx))
@@ -274,10 +273,17 @@ func loadCloudSettings(ctx *config.RunContext) {
 		logging.Logger.Info("Enabled actual costs")
 	}
 
-	if result.TagsAPIEnabled && ctx.Config.TagPolicyAPIEndpoint == "" {
-		ctx.Config.TagPolicyAPIEndpoint = ctx.Config.DashboardAPIEndpoint
+	if (result.PoliciesAPIEnabled || result.TagsAPIEnabled) && ctx.Config.PolicyV2APIEndpoint == "" {
+		ctx.Config.PolicyV2APIEndpoint = ctx.Config.DashboardAPIEndpoint
+		logging.Logger.Debug("Using default policies V2 endpoint")
 	}
-	if ctx.Config.TagPolicyAPIEndpoint != "" {
+
+	if result.PoliciesAPIEnabled {
+		ctx.Config.PoliciesEnabled = true
+		logging.Logger.Info("Enabled policies V2")
+	}
+
+	if result.TagsAPIEnabled {
 		ctx.Config.TagPoliciesEnabled = true
 		logging.Logger.Info("Enabled tag policies")
 	}
@@ -295,9 +301,8 @@ func checkAPIKey(apiKey string, apiEndpoint string, defaultEndpoint string) erro
 }
 
 var ignoredErrors = []string{
-	"Tag policy check failed",
 	"Policy check failed",
-	"Guardrail check failed",
+	"Governance check failed",
 }
 
 func handleCLIError(ctx *config.RunContext, cliErr error) {
