@@ -3,8 +3,6 @@ package main
 import (
 	"fmt"
 
-	log "github.com/sirupsen/logrus"
-
 	"github.com/spf13/cobra"
 
 	"github.com/infracost/infracost/internal/apiclient"
@@ -46,19 +44,23 @@ See https://infracost.io/docs/features/cli_commands/#upload-runs`,
 				return fmt.Errorf("could not load input file %s err: %w", path, err)
 			}
 
-			if ctx.Config.TagPolicyAPIEndpoint != "" {
-				tagPolicyClient := apiclient.NewTagPolicyAPIClient(ctx)
-				tagPolicies, err := tagPolicyClient.CheckTagPolicies(ctx, root)
+			if ctx.Config.PolicyV2APIEndpoint != "" {
+				policyClient, err := apiclient.NewPolicyAPIClient(ctx)
 				if err != nil {
-					log.WithError(err).Error("Failed to check tag policies")
-				}
+					logging.Logger.WithError(err).Error("Failed to initialize policies client")
+				} else {
+					policies, err := policyClient.CheckPolicies(ctx, root)
+					if err != nil {
+						logging.Logger.WithError(err).Error("Failed to check policies")
+					}
 
-				root.TagPolicies = tagPolicies
+					root.TagPolicies = policies.TagPolicies
+					root.FinOpsPolicies = policies.FinOpsPolicies
+				}
 			}
-			tagPolicyCheck := output.NewTagPolicyChecks(root.TagPolicies)
 
 			dashboardClient := apiclient.NewDashboardAPIClient(ctx)
-			result, err := dashboardClient.AddRun(ctx, root)
+			result, err := dashboardClient.AddRun(ctx, root, apiclient.CommentFormatMarkdownHTML)
 			if err != nil {
 				return fmt.Errorf("failed to upload to Infracost Cloud: %w", err)
 			}
@@ -75,12 +77,8 @@ See https://infracost.io/docs/features/cli_commands/#upload-runs`,
 				logging.Logger.WithError(err).Warn("could not report `infracost-upload` event")
 			}
 
-			if len(result.GuardrailCheck.BlockingFailures()) > 0 {
-				return result.GuardrailCheck.BlockingFailures()
-			}
-
-			if len(tagPolicyCheck.FailingTagPolicies) > 0 {
-				return tagPolicyCheck
+			if len(result.GovernanceFailures) > 0 {
+				return result.GovernanceFailures
 			}
 
 			return nil
