@@ -16,8 +16,14 @@ import (
 	sync2 "github.com/infracost/infracost/internal/sync"
 )
 
-func testLoaderE2E(t *testing.T, path string, expectedModules []*ManifestModule, sourceMap config.TerraformSourceMap, cleanup bool) {
-	if cleanup {
+type TestLoaderE2EOpts = struct {
+	SourceMap config.TerraformSourceMap
+	Cleanup   bool
+	IgnoreDir bool
+}
+
+func testLoaderE2E(t *testing.T, path string, expectedModules []*ManifestModule, opts TestLoaderE2EOpts) {
+	if opts.Cleanup {
 		err := os.RemoveAll(filepath.Join(path, config.InfracostDir))
 		assert.NoError(t, err)
 	}
@@ -25,7 +31,7 @@ func testLoaderE2E(t *testing.T, path string, expectedModules []*ManifestModule,
 	logger := logrus.New()
 	logger.SetOutput(io.Discard)
 
-	moduleLoader := NewModuleLoader(path, &CredentialsSource{FetchToken: credentials.FindTerraformCloudToken}, sourceMap, logrus.NewEntry(logger), &sync2.KeyMutex{})
+	moduleLoader := NewModuleLoader(path, &CredentialsSource{FetchToken: credentials.FindTerraformCloudToken}, opts.SourceMap, logrus.NewEntry(logger), &sync2.KeyMutex{})
 
 	manifest, err := moduleLoader.Load(path)
 	if !assert.NoError(t, err) {
@@ -36,13 +42,34 @@ func testLoaderE2E(t *testing.T, path string, expectedModules []*ManifestModule,
 		return expectedModules[i].Key < expectedModules[j].Key
 	})
 
+	expected := []*ManifestModule{}
+	for _, m := range expectedModules {
+		e := &ManifestModule{
+			Key:     m.Key,
+			Source:  m.Source,
+			Version: m.Version,
+		}
+
+		if !opts.IgnoreDir {
+			e.Dir = m.Dir
+		}
+
+		expected = append(expected, e)
+	}
+
 	actualModules := manifest.Modules
 
 	sort.Slice(actualModules, func(i, j int) bool {
 		return actualModules[i].Key < actualModules[j].Key
 	})
 
-	assert.Equal(t, expectedModules, actualModules)
+	if opts.IgnoreDir {
+		for _, m := range actualModules {
+			m.Dir = ""
+		}
+	}
+
+	assert.Equal(t, expected, actualModules)
 }
 
 func TestNestedModules(t *testing.T) {
@@ -90,7 +117,7 @@ func TestNestedModules(t *testing.T) {
 			Version: "3.4.0",
 			Dir:     ".infracost/terraform_modules/f8b5f5ddb85ee755b31c8b76d2801f5b",
 		},
-	}, config.TerraformSourceMap{}, true)
+	}, TestLoaderE2EOpts{Cleanup: true})
 }
 
 func TestSubmodules(t *testing.T) {
@@ -116,7 +143,7 @@ func TestSubmodules(t *testing.T) {
 			Source: "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git//modules/zones",
 			Dir:    ".infracost/terraform_modules/03c49f2fce2b8552355561b7ac4f2c94/modules/zones",
 		},
-	}, nil, true)
+	}, TestLoaderE2EOpts{Cleanup: true})
 }
 
 func TestModuleMultipleUses(t *testing.T) {
@@ -137,7 +164,7 @@ func TestModuleMultipleUses(t *testing.T) {
 			Version: "3.4.0",
 			Dir:     ".infracost/terraform_modules/f8b5f5ddb85ee755b31c8b76d2801f5b",
 		},
-	}, config.TerraformSourceMap{}, true)
+	}, TestLoaderE2EOpts{Cleanup: true})
 }
 
 func TestModuleMultipleUsesMissingManifest(t *testing.T) {
@@ -161,7 +188,7 @@ func TestModuleMultipleUsesMissingManifest(t *testing.T) {
 	}
 
 	// Run first time to download modules
-	testLoaderE2E(t, "./testdata/module_multiple_uses", expectedModules, config.TerraformSourceMap{}, true)
+	testLoaderE2E(t, "./testdata/module_multiple_uses", expectedModules, TestLoaderE2EOpts{Cleanup: false})
 
 	// Remove the manifest file to test we can still work with broken manifests
 	err := os.Remove("./testdata/module_multiple_uses/.infracost/terraform_modules/manifest.json")
@@ -170,7 +197,7 @@ func TestModuleMultipleUsesMissingManifest(t *testing.T) {
 	}
 
 	// Re-run without cleaning up the modules directory
-	testLoaderE2E(t, "./testdata/module_multiple_uses", expectedModules, config.TerraformSourceMap{}, false)
+	testLoaderE2E(t, "./testdata/module_multiple_uses", expectedModules, TestLoaderE2EOpts{Cleanup: false})
 }
 
 func TestWithCachedModules(t *testing.T) {
@@ -190,7 +217,7 @@ func TestWithCachedModules(t *testing.T) {
 			Source: "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git",
 			Dir:    ".infracost/terraform_modules/9740179dc58fea6ce4a32fdc5b4e0839",
 		},
-	}, config.TerraformSourceMap{}, false)
+	}, TestLoaderE2EOpts{Cleanup: false})
 
 	// Check that the modules were not overwritten
 	regModContents, err := os.ReadFile("./testdata/with_cached_modules/.infracost/terraform_modules/f8b5f5ddb85ee755b31c8b76d2801f5b/main.tf")
@@ -376,50 +403,368 @@ func TestMultiProject(t *testing.T) {
 	})
 }
 
-func TestWithSourceMap(t *testing.T) {
+func TestSourceMapRegistryModule(t *testing.T) {
 	if testing.Short() {
 		t.Skip("Skipping test in short mode")
 	}
 
-	sourceMap := config.TerraformSourceMap{
-		"terraform-aws-modules/ec2-instance/aws":                                                  "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v4.5.0",
-		"git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git":            "registry.terraform.io/terraform-aws-modules/ec2-instance/aws",
-		"git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v4.4.0": "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v4.3.0",
-		"terraform-aws-modules/route53/aws":                                                       "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git",
+	tests := []struct {
+		name      string
+		sourceMap config.TerraformSourceMap
+		want      []*ManifestModule
+	}{
+		{
+			name:      "empty source map",
+			sourceMap: config.TerraformSourceMap{},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-module",
+					Source:  "registry.terraform.io/terraform-aws-modules/ec2-instance/aws",
+					Version: "3.4.0",
+				},
+			},
+		},
+		{
+			name: "git without version",
+			sourceMap: config.TerraformSourceMap{
+				"terraform-aws-modules/ec2-instance/aws": "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:    "registry-module",
+					Source: "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git",
+					// We can't map the version here when going from registry to git, since we don't know which tag it should be.
+					// Some modules are prefixed with v and some aren't.
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git with version",
+			sourceMap: config.TerraformSourceMap{
+				"terraform-aws-modules/ec2-instance/aws": "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v5.5.0",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-module",
+					Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v5.5.0",
+					Version: "",
+				},
+			},
+		},
+		// This shouldn't map since the source in the code doesn't include the registry hostname
+		{
+			name: "with registry hostname",
+			sourceMap: config.TerraformSourceMap{
+				"registry.terraform.io/terraform-aws-modules/ec2-instance/aws": "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-module",
+					Source:  "registry.terraform.io/terraform-aws-modules/ec2-instance/aws",
+					Version: "3.4.0",
+				},
+			},
+		},
+		{
+			name: "with prefix",
+			sourceMap: config.TerraformSourceMap{
+				"terraform-aws-modules/": "registry.terraform.io/terraform-aws-modules/",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-module",
+					Source:  "registry.terraform.io/terraform-aws-modules/ec2-instance/aws",
+					Version: "3.4.0",
+				},
+			},
+		},
 	}
 
-	testLoaderE2E(t, "./testdata/modules_with_source_map", []*ManifestModule{
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			testLoaderE2E(t, "./testdata/sourcemap_registry_module", tt.want, TestLoaderE2EOpts{SourceMap: tt.sourceMap, Cleanup: true, IgnoreDir: true})
+		})
+	}
+}
+
+func TestSourceMapRegistrySubmodule(t *testing.T) {
+	if testing.Short() {
+		t.Skip("Skipping test in short mode")
+	}
+
+	tests := []struct {
+		name      string
+		sourceMap config.TerraformSourceMap
+		want      []*ManifestModule
+	}{
 		{
-			Key:     "registry-module",
-			Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v4.5.0",
-			Version: "",
-			Dir:     ".infracost/terraform_modules/90ed96c30cb769e39bbe5a2686ca87d1",
+			name:      "empty source map",
+			sourceMap: config.TerraformSourceMap{},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-submodule",
+					Source:  "registry.terraform.io/terraform-aws-modules/route53/aws//modules/zones",
+					Version: "2.5.0",
+				},
+			},
 		},
 		{
-			Key:     "git-module-with-ref",
-			Source:  "registry.terraform.io/terraform-aws-modules/ec2-instance/aws",
-			Version: "4.5.0",
-			Dir:     ".infracost/terraform_modules/571e67fb2167bf68f1bcb139b0a396cb",
+			name: "git without version",
+			sourceMap: config.TerraformSourceMap{
+				"terraform-aws-modules/route53/aws": "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:    "registry-submodule",
+					Source: "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git//modules/zones",
+					// We can't map the version here when going from registry to git, since we don't know which tag it should be.
+					// Some modules are prefixed with v and some aren't.
+					Version: "",
+				},
+			},
 		},
 		{
-			Key:     "git-module-with-mapped-ref",
-			Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v4.3.0",
-			Version: "",
-			Dir:     ".infracost/terraform_modules/1bef96b2e0948432531cc6631d07dee4",
+			name: "git with version",
+			sourceMap: config.TerraformSourceMap{
+				"terraform-aws-modules/route53/aws": "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git?ref=v2.10.2",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-submodule",
+					Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git//modules/zones?ref=v2.10.2",
+					Version: "",
+				},
+			},
+		},
+		// This shouldn't map since the source in the code doesn't include the registry hostname
+		{
+			name: "with registry hostname",
+			sourceMap: config.TerraformSourceMap{
+				"registry.terraform.io/terraform-aws-modules/route53/aws": "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-submodule",
+					Source:  "registry.terraform.io/terraform-aws-modules/route53/aws//modules/zones",
+					Version: "2.5.0",
+				},
+			},
 		},
 		{
-			Key:     "git-module-not-replaced",
-			Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-sns.git",
-			Version: "",
-			Dir:     ".infracost/terraform_modules/db69103dcf4b9586b710a97de31750bd",
+			name: "with prefix",
+			sourceMap: config.TerraformSourceMap{
+				"terraform-aws-modules/": "registry.terraform.io/terraform-aws-modules/",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "registry-submodule",
+					Source:  "registry.terraform.io/terraform-aws-modules/route53/aws//modules/zones",
+					Version: "2.5.0",
+				},
+			},
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			testLoaderE2E(t, "./testdata/sourcemap_registry_submodule", tt.want, TestLoaderE2EOpts{SourceMap: tt.sourceMap, Cleanup: true, IgnoreDir: true})
+		})
+	}
+}
+
+func TestSourceMapGitModule(t *testing.T) {
+	if testing.Short() {
+		t.Skip("Skipping test in short mode")
+	}
+
+	tests := []struct {
+		name      string
+		sourceMap config.TerraformSourceMap
+		want      []*ManifestModule
+	}{
+		{
+			name:      "empty source map",
+			sourceMap: config.TerraformSourceMap{},
+			want: []*ManifestModule{
+				{
+					Key:     "git-module",
+					Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v3.4.0",
+					Version: "",
+				},
+			},
 		},
 		{
-			Key:     "registry-module-with-submodule",
-			Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git//modules/zones",
-			Version: "",
-			Dir:     ".infracost/terraform_modules/5a4a225178eed58c86a47cd2229ec923/modules/zones",
+			name: "git without version",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git": "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-module",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git?ref=v3.4.0",
+					Version: "",
+				},
+			},
 		},
-	}, sourceMap, true)
+		{
+			name: "git with prefix",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/": "git::https://github.com/infracost-tests/",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-module",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git?ref=v3.4.0",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git with ref override",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git?ref=v3.4.0": "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git?ref=master",
+				"git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git":            "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-module",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git?ref=master",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git with repo override",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git": "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git",
+				"git::https://github.com/terraform-aws-modules/":                               "git::https://github.com/infracost-tests/",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-module",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-ec2-instance.git?ref=v3.4.0",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git to registry",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-ec2-instance.git": "registry.terraform.io/terraform-aws-modules/ec2-instance/aws",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-module",
+					Source:  "registry.terraform.io/terraform-aws-modules/ec2-instance/aws",
+					Version: "3.4.0",
+				},
+			},
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			testLoaderE2E(t, "./testdata/sourcemap_git_module", tt.want, TestLoaderE2EOpts{SourceMap: tt.sourceMap, Cleanup: true, IgnoreDir: true})
+		})
+	}
+}
+
+func TestSourceMapGitSubmodule(t *testing.T) {
+	if testing.Short() {
+		t.Skip("Skipping test in short mode")
+	}
+
+	tests := []struct {
+		name      string
+		sourceMap config.TerraformSourceMap
+		want      []*ManifestModule
+	}{
+		{
+			name:      "empty source map",
+			sourceMap: config.TerraformSourceMap{},
+			want: []*ManifestModule{
+				{
+					Key:     "git-submodule",
+					Source:  "git::https://github.com/terraform-aws-modules/terraform-aws-route53.git//modules/zones?ref=v2.5.0",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git without version",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-route53.git": "git::https://github.com/infracost-tests/terraform-aws-route53.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-submodule",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-route53.git//modules/zones?ref=v2.5.0",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git with prefix",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/": "git::https://github.com/infracost-tests/",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-submodule",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-route53.git//modules/zones?ref=v2.5.0",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git with ref override",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-route53.git?ref=v2.5.0": "git::https://github.com/infracost-tests/terraform-aws-route53.git?ref=master",
+				"git::https://github.com/terraform-aws-modules/terraform-aws-route53.git":            "git::https://github.com/infracost-tests/terraform-aws-route53.git",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-submodule",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-route53.git//modules/zones?ref=master",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git with repo override",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-route53.git": "git::https://github.com/infracost-tests/terraform-aws-route53.git",
+				"git::https://github.com/terraform-aws-modules/":                          "git::https://github.com/infracost-tests/",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-submodule",
+					Source:  "git::https://github.com/infracost-tests/terraform-aws-route53.git//modules/zones?ref=v2.5.0",
+					Version: "",
+				},
+			},
+		},
+		{
+			name: "git to registry",
+			sourceMap: config.TerraformSourceMap{
+				"git::https://github.com/terraform-aws-modules/terraform-aws-route53.git": "registry.terraform.io/terraform-aws-modules/route53/aws",
+			},
+			want: []*ManifestModule{
+				{
+					Key:     "git-submodule",
+					Source:  "registry.terraform.io/terraform-aws-modules/route53/aws//modules/zones",
+					Version: "2.5.0",
+				},
+			},
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			testLoaderE2E(t, "./testdata/sourcemap_git_submodule", tt.want, TestLoaderE2EOpts{SourceMap: tt.sourceMap, Cleanup: true, IgnoreDir: true})
+		})
+	}
 }
 
 func assertModulesEqual(t *testing.T, moduleLoader *ModuleLoader, path string, expectedModules []*ManifestModule) {
