@@ -126,11 +126,13 @@ func CompareTo(c *config.Config, current, prior Root) (Root, error) {
 		scp := p.ToSchemaProject()
 		scp.Diff = scp.Resources
 		scp.PastResources = nil
+		scp.Metadata.PastPolicySha = ""
 		scp.HasDiff = true
 
 		if v, ok := priorProjects[p.LabelWithMetadata()]; ok {
 			if !p.Metadata.HasErrors() && !v.Metadata.HasErrors() {
 				scp.PastResources = v.Resources
+				scp.Metadata.PastPolicySha = v.Metadata.PolicySha
 				scp.Diff = schema.CalculateDiff(scp.PastResources, scp.Resources)
 			}
 
@@ -157,6 +159,7 @@ func CompareTo(c *config.Config, current, prior Root) (Root, error) {
 	for _, scp := range priorProjects {
 		scp.PastResources = scp.Resources
 		scp.Resources = nil
+		scp.Metadata.PolicySha = ""
 		scp.HasDiff = true
 		scp.Diff = schema.CalculateDiff(scp.PastResources, scp.Resources)
 
@@ -202,6 +205,7 @@ func Combine(inputs []ReportInput) (Root, error) {
 	projects := make([]Project, 0)
 	summaries := make([]*Summary, 0, len(inputs))
 	var tagPolicies []TagPolicy
+	var finOpsPolicies []FinOpsPolicy
 	currency := ""
 
 	var metadata Metadata
@@ -226,6 +230,10 @@ func Combine(inputs []ReportInput) (Root, error) {
 			tagPolicies = append(tagPolicies, input.Root.TagPolicies...)
 		}
 
+		if len(input.Root.FinOpsPolicies) > 0 {
+			finOpsPolicies = append(finOpsPolicies, input.Root.FinOpsPolicies...)
+		}
+
 		if input.Root.TotalHourlyCost != nil {
 			if totalHourlyCost == nil {
 				totalHourlyCost = decimalPtr(decimal.Zero)
@@ -291,6 +299,7 @@ func Combine(inputs []ReportInput) (Root, error) {
 	combined.Summary = MergeSummaries(summaries)
 	combined.Metadata = metadata
 	combined.TagPolicies = mergeTagPolicies(tagPolicies)
+	combined.FinOpsPolicies = mergeFinOpsPolicies(finOpsPolicies)
 	if len(inputs) > 0 {
 		combined.CloudURL = inputs[len(inputs)-1].Root.CloudURL
 	}
@@ -330,6 +339,30 @@ func mergeTagPolicies(tagPolicies []TagPolicy) []TagPolicy {
 	return tpMerged
 }
 
+func mergeFinOpsPolicies(finOpsPolicies []FinOpsPolicy) []FinOpsPolicy {
+	// gather and merge tag policies by id
+	fpMap := map[string]FinOpsPolicy{}
+	for _, fp := range finOpsPolicies {
+		if existingFp, ok := fpMap[fp.PolicyID]; ok {
+			fp.PrComment = existingFp.PrComment || fp.PrComment
+			fp.BlockPr = existingFp.BlockPr || fp.BlockPr
+			fp.Resources = append(existingFp.Resources, fp.Resources...)
+		}
+		fpMap[fp.PolicyID] = fp
+	}
+
+	fpMerged := make([]FinOpsPolicy, 0, len(fpMap))
+	// use the original tagPolicies array to iterate over the map so the order is preserved
+	for _, fp := range finOpsPolicies {
+		if mergedFp, ok := fpMap[fp.PolicyID]; ok {
+			fpMerged = append(fpMerged, mergedFp)
+			delete(fpMap, fp.PolicyID)
+		}
+	}
+
+	return fpMerged
+}
+
 func checkCurrency(inputCurrency, fileCurrency string) (string, error) {
 	if fileCurrency == "" {
 		fileCurrency = "USD" // default to USD
