@@ -62,21 +62,21 @@ func formatCostChangeSentence(currency string, pastCost, cost *decimal.Decimal,
 	}
 
 	if pastCost == nil {
-		return "monthly cost will increase by " + formatCost(currency, cost) + " " + up
+		return "Monthly cost will increase by " + formatCost(currency, cost) + " " + up
 	}
 
 	diff := cost.Sub(*pastCost).Abs()
 	change := formatCost(currency, &diff)
 
 	if pastCost.Equals(*cost) {
-		return "monthly cost will not change"
+		return "Monthly cost will not change"
 	}
 
 	if pastCost.GreaterThan(*cost) {
-		return "monthly cost will decrease by " + change + " " + down
+		return "Monthly cost will decrease by " + change + " " + down
 	}
 
-	return "monthly cost will increase by " + change + " " + up
+	return "Monthly cost will increase by " + change + " " + up
 }
 
 func calculateMetadataToDisplay(projects []Project) (hasModulePath bool, hasWorkspace bool) {
