@@ -8,6 +8,8 @@ import (
 	"github.com/infracost/infracost/internal/usage"
 	"github.com/shopspring/decimal"
 	"github.com/tidwall/gjson"
+	"golang.org/x/text/cases"
+	"golang.org/x/text/language"
 )
 
 func GetAzureRMSearchServiceRegistryItem() *schema.RegistryItem {
@@ -54,7 +56,7 @@ func NewAzureRMSearchService(d *schema.ResourceData, u *schema.UsageData) *schem
 	var skuName string
 	skuElems := strings.Split(sku, " ")
 	for _, v := range skuElems {
-		skuName += strings.Title(v) + " "
+		skuName += cases.Title(language.English).String(v) + " "
 	}
 	unitName := "unit"
 	if units.GreaterThan(decimal.NewFromInt(1)) {
