@@ -1,6 +1,7 @@
 package azure
 
 import (
+	"fmt"
 	"regexp"
 	"strings"
 
@@ -18,6 +19,10 @@ func decimalPtr(d decimal.Decimal) *decimal.Decimal {
 	return &d
 }
 
+func regexPtr(regex string) *string {
+	return strPtr(fmt.Sprintf("/%s/i", regex))
+}
+
 var sReg = regexp.MustCompile(`\s+`)
 
 func toAzureCLIName(location string) string {
