@@ -4,5 +4,5 @@ type Provider interface {
 	Type() string
 	DisplayType() string
 	AddMetadata(*ProjectMetadata)
-	LoadResources(map[string]*UsageData) ([]*Project, error)
+	LoadResources(UsageMap) ([]*Project, error)
 }
